package probes::IOSPing;

=head1 NAME

probes::IOSPing - Cisco IOS Probe for SmokePing

=head1 SYNOPSIS

 *** Probes ***
 + IOSPing
 binary = /usr/bin/remsh
 packetsize = 1024

 ++ PROBE_CONF
 ioshost = router

=head1 DESCRIPTION

Integrates Cisco IOS as a probe into smokeping.

The (optional) packetsize option lets you configure the packetsize for
the pings sent.

The binary and ioshost options are mandatory.  The ioshost option
specifies the Cisco IOS router which should be used for the ping.

The FPing manpage has the following to say on this toppic:

Number of bytes of ping data to send.  The minimum size (normally 12) allows
room for the data that fping needs to do its work (sequence number,
timestamp).  The reported received data size includes the IP header
(normally 20 bytes) and ICMP header (8 bytes), so the minimum total size is
40 bytes.  Default is 56, as in ping. Maximum is the theoretical maximum IP
datagram size (64K), though most systems limit this to a smaller,
system-dependent number.

=head1 AUTHOR

Paul J Murphy <paul@murph.org>

based on probes::FPing by

Tobias Oetiker <tobi@oetiker.ch>

=cut

use strict;
use base qw(probes::basefork);
use IPC::Open2;
use Symbol;
use Carp;

sub new($$$)
{
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = $class->SUPER::new(@_);

    # no need for this if we run as a cgi
    unless ( $ENV{SERVER_SOFTWARE} ) {
        croak "ERROR: IOSPing packetsize must be between 12 and 64000"
           if $self->{properties}{packetsize} and 
              ( $self->{properties}{packetsize} < 12 or $self->{properties}{packetsize} > 64000 ); 

        croak "ERROR: IOSPing 'binary' not defined in IOSPing probe definition"
            unless defined $self->{properties}{binary};

        croak "ERROR: IOSPing 'binary' does not point to an executable"
            unless -f $self->{properties}{binary} and -x $self->{properties}{binary};

	$self->{pingfactor} = 1000; # Gives us a good-guess default
	print "### assuming you are using an fping copy reporting in miliseconds\n";
    };

    return $self;
}

sub ProbeDesc($){
    my $self = shift;
    my $bytes = $self->{properties}{packetsize} || 56;
    return "ICMP Echo Pings ($bytes Bytes)";
}

sub pingone ($$){
    my $self = shift;
    my $target = shift;
    my $bytes = $self->{properties}{packetsize} || 56;
    # do NOT call superclass ... the ping method MUST be overwriten
    my %upd;
    my $inh = gensym;
    my $outh = gensym;

    croak "ERROR: IOSPing 'ioshost' not defined"
	unless defined $target->{vars}{ioshost};

    my $pid = open2($outh,$inh,
		    $self->{properties}{binary},
		    $target->{vars}{ioshost},
		    'ping');
# Protocol [ip]: 
    print { $inh } "\n";
# Target IP address: 
    print { $inh } $target->{addr},"\n";
# Repeat count [5]: 
    print { $inh } $self->{cfg}{Database}{pings},"\n";
# Datagram size [100]: 
    print { $inh } $bytes,"\n";
# Timeout in seconds [2]: 1
    print { $inh } "\n";
# Extended commands [n]: y
    print { $inh } "y\n";
# Source address or interface: 
    print { $inh } "\n";
# Type of service [0]: 
    print { $inh } "\n";
# Set DF bit in IP header? [no]: 
    print { $inh } "\n";
# Validate reply data? [no]: 
    print { $inh } "\n";
# Data pattern [0xABCD]: 
    print { $inh } "\n";
# Loose, Strict, Record, Timestamp, Verbose[none]: V
    print { $inh } "V\n";
# Loose, Strict, Record, Timestamp, Verbose[V]: 
    print { $inh } "\n";
# Sweep range of sizes [n]: 
    print { $inh } "\n";

    my @times = ();
    while (<$outh>){
	chomp;
	/^Reply to request \d+ \((\d+) ms\)/ && push(@times,$1);
    }
    @times = map {sprintf "%.10e", $_ / $self->{pingfactor}} sort {$a <=> $b} @times;

    waitpid $pid,0;
    close $inh;
    close $outh;

    return @times;
}

1;
